@@ -98,8 +98,8 @@ class ProductInsightResource:
         server_domain: Optional[str] = req.get_param("server_domain")
         response: JSONType = {}
         insights = [
-            i.serialize()
-            for i in get_insights(
+            insight.to_dict()
+            for insight in get_insights(
                 barcode=barcode, server_domain=server_domain, limit=None
             )
         ]
@@ -137,6 +137,7 @@ class InsightCollection:
         annotation: Optional[int] = req.get_param_as_int("annotation")
         value_tag: str = req.get_param("value_tag")
         brands = req.get_param_as_list("brands") or None
+        predictor = req.get_param("predictor")
         server_domain: Optional[str] = req.get_param("server_domain")
 
         if keep_types:
@@ -157,6 +158,7 @@ class InsightCollection:
             annotated=annotated,
             annotation=annotation,
             barcode=barcode,
+            predictor=predictor,
         )
 
         offset: int = (page - 1) * count
@@ -181,7 +183,8 @@ class RandomInsightResource:
         country: Optional[str] = req.get_param("country")
         value_tag: Optional[str] = req.get_param("value_tag")
         server_domain: Optional[str] = req.get_param("server_domain")
-        count: int = req.get_param_as_int("count", default=1, min_value=1, max_value=50)
+        count: int = req.get_param_as_int("count", min_value=1, default=25)
+        predictor = req.get_param("predictor")
 
         keep_types = [insight_type] if insight_type else None
         get_insights_ = functools.partial(
@@ -191,9 +194,10 @@ class RandomInsightResource:
             value_tag=value_tag,
             order_by="random",
             server_domain=server_domain,
+            predictor=predictor,
         )
 
-        insights = [i.serialize() for i in get_insights_(limit=count)]
+        insights = [insight.to_dict() for insight in get_insights_(limit=count)]
         response["count"] = get_insights_(count=True)
 
         if not insights:
@@ -252,16 +256,14 @@ class AnnotateInsightResource:
         # This field is only needed for nutritional table structure insights.
         data = req.get_param_as_json("data")
 
-        auth: Optional[OFFAuthentication] = parse_auth(req)
-        trusted_annotator: bool = auth is not None
+        auth = parse_auth(req)
+        trusted_annotator = auth is not None
 
         device_id = device_id_from_request(req)
-
-        username = auth.get_username() if auth else "unknown annotator"
         logger.info(
-            "New annotation received from {} (annotation: {}, insight: {})".format(
-                username, annotation, insight_id
-            )
+            "New annotation received from "
+            f"{auth.get_username() if auth else 'unknown annotator'} "
+            f"(annotation: {annotation}, insight: {insight_id})"
         )
 
         annotation_result = save_annotation(
@@ -679,6 +681,9 @@ class ImageLogoSearchResource:
         if annotated is not None:
             where_clauses.append(LogoAnnotation.annotation_value.is_null(not annotated))
 
+        if min_confidence is not None:
+            where_clauses.append(LogoAnnotation.score >= min_confidence)
+
         join_image_prediction = False
         join_image_model = False
 
@@ -686,12 +691,6 @@ class ImageLogoSearchResource:
             where_clauses.append(ImageModel.server_domain == server_domain)
             join_image_model = True
 
-        if min_confidence is not None:
-            # TODO(raphael): We should filter based on individual logo object confidence
-            # and not on image object with the maximum confidence
-            where_clauses.append(ImagePrediction.max_confidence >= min_confidence)
-            join_image_prediction = True
-
         if barcode is not None:
             where_clauses.append(ImageModel.barcode == barcode)
             join_image_model = True
@@ -914,14 +913,14 @@ class WebhookProductResource:
 
 
 class ProductQuestionsResource:
-    """Get a question about a product to confirm/infirm an insight
+    """Get questions about a product to confirm/infirm an insight
 
     see also doc/explanation/questions.md
     """
 
     def on_get(self, req: falcon.Request, resp: falcon.Response, barcode: str):
         response: JSONType = {}
-        count: int = req.get_param_as_int("count", min_value=1) or 1
+        count: int = req.get_param_as_int("count", min_value=1, default=25)
         lang: str = req.get_param("lang", default="en")
         # If the device_id is not provided as a request parameter, we use the
         # hash of the IPs as a backup.
@@ -991,6 +990,7 @@ def get_questions_resource_on_get(
     )
     # filter by annotation campaign
     campaign: Optional[str] = req.get_param("campaign")
+    predictor = req.get_param("predictor")
 
     # If the device_id is not provided as a request parameter, we use the
     # hash of the IPs as a backup.
@@ -1024,6 +1024,7 @@ def get_questions_resource_on_get(
         avoid_voted_on=_get_skip_voted_on(auth, device_id),
         automatically_processable=False,
         campaign=campaign,
+        predictor=predictor,
     )
 
     offset: int = (page - 1) * count
@@ -1214,6 +1215,7 @@ class UnansweredQuestionCollection:
             "reserved_barcode", default=False
         )
         campaign: Optional[str] = req.get_param("campaign")
+        predictor = req.get_param("predictor")
 
         get_insights_ = functools.partial(
             get_insights,
@@ -1225,6 +1227,7 @@ class UnansweredQuestionCollection:
             automatically_processable=False,
             reserved_barcode=reserved_barcode,
             campaign=campaign,
+            predictor=predictor,
         )
 
         offset: int = (page - 1) * count
