@@ -24,7 +24,12 @@ from robotoff.products import (
     get_product_store,
     is_valid_image,
 )
-from robotoff.taxonomy import Taxonomy, get_taxonomy
+from robotoff.taxonomy import (
+    Taxonomy,
+    TaxonomyType,
+    get_taxonomy,
+    match_taxonomized_value,
+)
 from robotoff.utils import get_logger, text_file_iter
 from robotoff.utils.cache import CachedStore
 
@@ -272,7 +277,6 @@ class InsightImporter(metaclass=abc.ABCMeta):
         cls,
         predictions: List[Prediction],
         server_domain: str,
-        automatic: bool,
         product_store: DBProductStore,
     ) -> int:
         """Import insights, this is the main method.
@@ -285,8 +289,8 @@ class InsightImporter(metaclass=abc.ABCMeta):
                 raise ValueError(f"unexpected prediction type: '{prediction.type}'")
 
         inserts = 0
-        for to_create, to_delete in cls.generate_insights(
-            predictions, server_domain, automatic, product_store
+        for to_create, to_update, to_delete in cls.generate_insights(
+            predictions, server_domain, product_store
         ):
             if to_delete:
                 to_delete_ids = [insight.id for insight in to_delete]
@@ -301,6 +305,9 @@ class InsightImporter(metaclass=abc.ABCMeta):
                     50,
                 )
 
+            for insight in to_update:
+                insight.save()
+
         return inserts
 
     @classmethod
@@ -308,11 +315,12 @@ class InsightImporter(metaclass=abc.ABCMeta):
         cls,
         predictions: List[Prediction],
         server_domain: str,
-        automatic: bool,
         product_store: DBProductStore,
-    ) -> Iterator[Tuple[List[ProductInsight], List[ProductInsight]]]:
+    ) -> Iterator[
+        Tuple[List[ProductInsight], List[ProductInsight], List[ProductInsight]]
+    ]:
         """Given a list of predictions, yield tuples of ProductInsight to
-        create and delete.
+        create, update and delete.
 
         It calls the `generate_candidates` method, specific to each insight type
         (and implemented in sub-classes).
@@ -332,7 +340,7 @@ class InsightImporter(metaclass=abc.ABCMeta):
                     f"Product {barcode} not found in DB, deleting existing insights"
                 )
                 if references:
-                    yield [], references
+                    yield [], [], references
                 continue
 
             product_predictions = sort_predictions(group)
@@ -362,14 +370,23 @@ class InsightImporter(metaclass=abc.ABCMeta):
                     # Note: we could add vote annotation for anonymous user,
                     # but it should be done outside this loop. It's not yet implemented
 
-            to_create, to_delete = cls.get_insight_update(candidates, references)
+            to_create, to_update, to_delete = cls.get_insight_update(
+                candidates, references
+            )
 
             for insight in to_create:
-                if not automatic:
-                    insight.automatic_processing = False
                 cls.add_fields(insight, product, timestamp, server_domain, server_type)
 
-            yield to_create, to_delete
+            for insight, reference_insight in to_update:
+                # Keep `reference_insight` in DB (as the value/value_tag/source_image is the same),
+                # but update information from `insight`.
+                # This way, we don't unnecessarily insert/delete rows in ProductInsight table
+                # and we keep associated votes
+                cls.update_fields(insight, reference_insight, product, timestamp)
+
+            yield to_create, [
+                reference_insight for (_, reference_insight) in to_update
+            ], to_delete
 
     @classmethod
     @abc.abstractmethod
@@ -394,20 +411,31 @@ class InsightImporter(metaclass=abc.ABCMeta):
     @classmethod
     def get_insight_update(
         cls, candidates: List[ProductInsight], reference_insights: List[ProductInsight]
-    ) -> Tuple[List[ProductInsight], List[ProductInsight]]:
+    ) -> Tuple[
+        List[ProductInsight],
+        List[Tuple[ProductInsight, ProductInsight]],
+        List[ProductInsight],
+    ]:
         """Return a tuple containing:
-        - a list of ProductInsight to import
+        - a list of ProductInsight to create
+        - a list of ProductInsight to update, as (insight, reference_insight)
+          tuples, where `insight` is the candidate and `reference_insight` is
+          the insight already in DB
         - a list of ProductInsight to delete
 
         :param candidates: candidate predictions
         :param reference_insights: existing insights of this type and product
         """
-        to_create: List[ProductInsight] = []
+        to_create_or_update: List[Tuple[ProductInsight, Optional[ProductInsight]]] = []
         # Keep already annotated insights in DB
         to_keep_ids = set(
             reference.id
             for reference in reference_insights
+            # Don't delete already annotated insights
             if reference.annotation is not None
+            # Don't overwrite an insight that is going to be applied
+            # automatically soon
+            or reference.automatic_processing is True
         )
         for candidate in sort_candidates(candidates):
             # if match is True, candidate conflicts with existing insight,
@@ -416,21 +444,53 @@ class InsightImporter(metaclass=abc.ABCMeta):
                 cls.is_conflicting_insight(candidate, reference_insight)
                 for reference_insight in reference_insights
                 if reference_insight.annotation is not None
+                # Don't overwrite an insight that is going to be applied
+                # automatically soon
+                or reference_insight.automatic_processing is True
             )
 
             if not match:
-                for selected in to_create:
+                for selected, _ in to_create_or_update:
                     if cls.is_conflicting_insight(candidate, selected):
                         # Don't import candidate if it conflicts with an
                         # already selected candidate
                         break
                 else:
-                    to_create.append(candidate)
+                    mapping_ref_insight = None
+                    # In order for the voting system to work, we map insights to create
+                    # to existing insights with the same value/value_tag/source_image.
+                    # This way, we don't loose vote information.
+                    for reference_insight in reference_insights:
+                        if (
+                            reference_insight.annotation is None
+                            and cls.is_conflicting_insight(candidate, reference_insight)
+                            # only map to existing insight if the source image is the same,
+                            # otherwise create a new insight
+                            and candidate.source_image == reference_insight.source_image
+                        ):
+                            mapping_ref_insight = reference_insight
+                            to_keep_ids.add(reference_insight.id)
+                            break
+
+                    # If mapping_ref_insight is None, a new insight is created in DB,
+                    # Otherwise the reference insight is updated with candidate
+                    # information
+                    to_create_or_update.append((candidate, mapping_ref_insight))
 
         to_delete = [
             insight for insight in reference_insights if insight.id not in to_keep_ids
         ]
-        return to_create, to_delete
+        to_create = [
+            insight
+            for insight, ref_insight in to_create_or_update
+            if ref_insight is None
+        ]
+        to_update = [
+            (insight, ref_insight)
+            for insight, ref_insight in to_create_or_update
+            if ref_insight is not None
+        ]
+        return to_create, to_update, to_delete
 
     @classmethod
     @abc.abstractmethod
@@ -466,6 +526,7 @@ class InsightImporter(metaclass=abc.ABCMeta):
         insight.countries = product.countries_tags
         insight.brands = product.brands_tags
         insight.n_votes = 0
+        insight.unique_scans_n = product.unique_scans_n
 
         if insight.automatic_processing:
             insight.process_after = timestamp + datetime.timedelta(
@@ -474,6 +535,36 @@ class InsightImporter(metaclass=abc.ABCMeta):
 
         cls.add_optional_fields(insight, product)
 
+    @classmethod
+    def update_fields(
+        cls,
+        insight: ProductInsight,
+        reference_insight: ProductInsight,
+        product: Product,
+        timestamp: datetime.datetime,
+    ):
+        """Update `reference_insight` fields with data from `insight`.
+
+        This is used to refresh `reference_insight` with potentially fresher
+        information, while avoiding row deletion/insertion each time
+        `import_insights` is called.
+        """
+        cls.add_fields(
+            insight,
+            product,
+            timestamp,
+            reference_insight.server_domain,
+            reference_insight.server_type,
+        )
+
+        for field_name in (
+            key
+            for key in insight.__data__.keys()
+            if key not in ("id", "barcode", "type", "server_domain", "server_type")
+        ):
+            if getattr(insight, field_name) != getattr(reference_insight, field_name):
+                setattr(reference_insight, field_name, getattr(insight, field_name))
+
     @classmethod
     def add_optional_fields(cls, insight: ProductInsight, product: Product):
         """Overwrite this method in children classes to add optional fields.
@@ -506,9 +597,10 @@ class PackagerCodeInsightImporter(InsightImporter):
         product: Product,
         emb_code: str,
     ) -> bool:
-        return normalize_emb_code(emb_code) not in [
-            normalize_emb_code(c) for c in product.emb_codes_tags
-        ]
+        existing_codes = [normalize_emb_code(c) for c in product.emb_codes_tags]
+        normalized_code = normalize_emb_code(emb_code)
+
+        return normalized_code not in existing_codes
 
     @classmethod
     def generate_candidates(
@@ -566,6 +658,17 @@ class LabelInsightImporter(InsightImporter):
             for prediction in predictions
             if cls.is_prediction_valid(product, prediction.value_tag)  # type: ignore
         ]
+        for candidate in candidates:
+            if candidate.value_tag:
+                # we normalize `value_tag` to the canonical taxonomy value
+                # it helps to deal with internationalization
+                # (`fr:sans-gluten` and `en:no-gluten` is the same tag)
+                value_tag = match_taxonomized_value(
+                    candidate.value_tag, TaxonomyType.label.name
+                )
+                if value_tag is not None:
+                    candidate.value_tag = value_tag
+
         taxonomy = get_taxonomy(InsightType.label.name)
         for candidate in select_deepest_taxonomized_candidates(candidates, taxonomy):
             insight = ProductInsight(**candidate.to_dict())
@@ -945,7 +1048,6 @@ IMPORTERS: List[Type[InsightImporter]] = [
 def import_insights(
     predictions: Iterable[Prediction],
     server_domain: str,
-    automatic: bool,
     product_store: Optional[DBProductStore] = None,
 ) -> int:
     if product_store is None:
@@ -955,14 +1057,13 @@ def import_insights(
         predictions, product_store, server_domain
     )
     return import_insights_for_products(
-        updated_prediction_types_by_barcode, server_domain, automatic, product_store
+        updated_prediction_types_by_barcode, server_domain, product_store
     )
 
 
 def import_insights_for_products(
     prediction_types_by_barcode: Dict[str, Set[PredictionType]],
     server_domain: str,
-    automatic: bool,
     product_store: DBProductStore,
 ) -> int:
     """Re-compute insights for products with new predictions.
@@ -987,7 +1088,7 @@ def import_insights_for_products(
                 )
             ]
             imported += importer.import_insights(
-                predictions, server_domain, automatic, product_store
+                predictions, server_domain, product_store
             )
     return imported
 
@@ -1031,7 +1132,6 @@ def import_predictions(
 def refresh_insights(
     barcode: str,
     server_domain: str,
-    automatic: bool,
     product_store: Optional[DBProductStore] = None,
 ) -> int:
     """Refresh all insights for specific product.
@@ -1046,7 +1146,6 @@ def refresh_insights(
 
     :param barcode: Barcode of the product.
     :param server_domain: The server domain associated with the predictions.
-    :param automatic: If False, no insight is applied automatically.
     :param product_store: The product store to use, defaults to None
     :return: The number of imported insights.
     """
@@ -1063,7 +1162,6 @@ def refresh_insights(
             imported += importer.import_insights(
                 [p for p in predictions if p.type in required_prediction_types],
                 server_domain,
-                automatic,
                 product_store,
             )
 
@@ -1072,22 +1170,20 @@ def refresh_insights(
 
 def refresh_all_insights(
     server_domain: str,
-    automatic: bool,
     product_store: Optional[DBProductStore] = None,
 ):
     """Refresh insights of all products for which we have predictions.
 
     :param server_domain: The server domain associated with the predictions.
-    :param automatic: If False, no insight is applied automatically.
     :param product_store: The product store to use, defaults to None
     :return: The number of imported insights.
     """
     imported = 0
-    for barcode in (
-        PredictionModel.select(fn.Distinct(PredictionModel.barcode)).scalar().iterator()
+    for (barcode,) in (
+        PredictionModel.select(fn.Distinct(PredictionModel.barcode)).tuples().iterator()
     ):
         logger.info(f"Refreshing insights for product {barcode}")
-        imported += refresh_insights(barcode, server_domain, automatic, product_store)
+        imported += refresh_insights(barcode, server_domain, product_store)
 
     return imported
 
