@@ -1,3 +1,5 @@
+import threading
+import time
 from multiprocessing.connection import Listener
 from multiprocessing.pool import Pool
 from typing import Dict
@@ -13,7 +15,17 @@ settings.init_sentry()
 logger = get_logger()
 
 
+def send_task_to_pool(pool, event_type, event_kwargs, delay):
+    """Simply pass the task to a worker in the pool, while eventually applying a delay"""
+    if delay:
+        time.sleep(delay)
+    logger.debug("Sending task to pool...")
+    pool.apply_async(run_task, (event_type, event_kwargs))
+    logger.debug("Task sent")
+
+
 def run():
+    """This is the event listener, it will receive task requests and launch them"""
     pool: Pool = Pool(settings.WORKER_COUNT, maxtasksperchild=30)
 
     logger.info("Starting listener server on {}:{}".format(*settings.IPC_ADDRESS))
@@ -32,8 +44,14 @@ def run():
                     logger.info(f"New '{event_type}' event received")
                     event_kwargs: Dict = event.get("meta", {})
 
-                    logger.debug("Sending task to pool...")
-                    pool.apply_async(run_task, (event_type, event_kwargs))
-                    logger.debug("Task sent")
+                delay = event_kwargs.pop("task_delay", None)
+                args = [pool, event_type, event_kwargs, delay]
+                if delay:
+                    # we have a delay, so spend it in a thread instead of listener main thread
+                    threading.Thread(target=send_task_to_pool, args=args).start()
+                else:
+                    # direct call, it's fast
+                    send_task_to_pool(*args)
+
             except Exception:
                 capture_exception()
